/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


/*
 * Filename: VersionInfo.cpp
 * Description: Bible version info
 * Ported from palmbible plus (http://palmbibleplus.sourceforge.net)
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
#include "util.h"
#include "VersionInfo.h"


EXPORT_C VersionInfo::VersionInfo(Database *dbase)
{
	this->dbase = dbase;
	infVersion = 1;
}

EXPORT_C VersionInfo::~VersionInfo()
{
	//closeallrecord
	if (infVersionInfoPtr) {
		unlockVersionPtr();
	}
}

int VersionInfo::GetNextBook(int currentbook)
{
	lockVersionPtr();
	for (int i  = 0 ; i < infVersionInfoPtr->totalBooks; i++ ){
		BookInfoPtr bookInfo = &infVersionInfoPtr->bookInfo[i];
		if (bookInfo->bookNum == currentbook) {
			if (i+1==infVersionInfoPtr->totalBooks) {
			  unlockVersionPtr();
				return -1;
			} else {
			  int next = infVersionInfoPtr->bookInfo[i+1].bookNum;
			  unlockVersionPtr();
			  return 	next;
			}
		}
	}
	unlockVersionPtr();
	return -1;
}

int VersionInfo::GetFirstBook()
{
	int booknum = 10;
	lockVersionPtr();
	if (infVersionInfoPtr->totalBooks>0) {
		booknum = infVersionInfoPtr->bookInfo[0].bookNum;
	}
	unlockVersionPtr();
	return booknum;
}

int VersionInfo::GetPrevBook(int currentbook)
{
	lockVersionPtr();
	for (int i  = 0 ; i < infVersionInfoPtr->totalBooks; i++ ){
		BookInfoPtr bookInfo = &infVersionInfoPtr->bookInfo[i];
		if (bookInfo->bookNum == currentbook) {
			if (i==0) {
				unlockVersionPtr();
				return -1;
			} else {
				int booknum = infVersionInfoPtr->bookInfo[i-1].bookNum;
				unlockVersionPtr();
				return 	booknum;
			}
		}
	}
	unlockVersionPtr();
	return -1;
}


int VersionInfo::GetBookIndex(char *bookname)
{

#ifdef USE_LIB_C
	unsigned int booklen = strlen(bookname);
#else
	unsigned int booklen = User::StringLength((const TUint8*)bookname);
#endif
	lockVersionPtr();
	for (int i  = 0 ; i < infVersionInfoPtr->totalBooks; i++ ){
		BookInfoPtr bookInfo = &infVersionInfoPtr->bookInfo[i];

#ifdef USE_LIB_C
		if ((unsigned int)strlen(bookInfo->complexName)<booklen) continue;
#else
		if ((unsigned int)User::StringLength((const TUint8*)bookInfo->complexName)<booklen) continue;
#endif
		unsigned int j;
		for (j=0; j<booklen; j++){
			if (my_tolower(bookInfo->complexName[j])!=bookname[j]) break;
		}
		if (j==booklen) {
			int num = bookInfo->bookNum;
			unlockVersionPtr();
			return num;
		}
	}
	unlockVersionPtr();
	return 0;
}

void VersionInfo::lockVersionPtr()
{
	char versionname[16], versioninfo[128], sepchar;
	unsigned char versionattr;
	unsigned short wordIndex, totalWordRec, totalBooks;

	char *s = (char *)dbase->DmxLockVersionRecord(infVersion, infVerInfoRecord);

    //infVersionInfoPtr = (VersionInfoPtr)
	int idx = 0;
	Mem::Copy(versionname, s + idx, 16);
	idx += 16;
	Mem::Copy(versioninfo, s + idx, 128);
	idx += 128;
	Mem::Copy(&sepchar, s + idx, 1);
	idx += 1;
	Mem::Copy(&versionattr, s + idx, 1);
	idx += 1;
	Mem::Copy(&wordIndex, s + idx, 2);
	idx += 2;
	Mem::Copy(&totalWordRec, s + idx, 2);
	idx += 2;
	Mem::Copy(&totalBooks, s + idx, 2);
	idx += 2;

#ifdef USE_LIB_C	
	infVersionInfoPtr = (VersionInfoPtr) User::Alloc(sizeof(VersionInfoType));
#else
	infVersionInfoPtr = (VersionInfoPtr) User::AllocLC(sizeof(VersionInfoType));
#endif

        infVersionInfoPtr->wordIndex = fixs(wordIndex);
	infVersionInfoPtr->sepChar = sepchar;
	
	infVersionInfoPtr->versionAttrib = versionattr;
        infVersionInfoPtr->totalWordRec= fixs(totalWordRec);
        infVersionInfoPtr->totalBooks= fixs(totalBooks);
        infVersionInfoPtr->bookInfo  = (BookInfoPtr)User::AllocL(sizeof(BookInfoType)*infVersionInfoPtr->totalBooks);
#ifndef USE_LIB_C
	CleanupStack::Pop(infVersionInfoPtr);
#endif
	
	for (int i  = 0 ; i < infVersionInfoPtr->totalBooks; i++ ){
		unsigned short bookNum, bookIndex, totalBookRec;
		char simpleName[8];
        	char complexName[32];

		Mem::Copy(&bookNum, s + idx, 2);
		idx += 2;
		Mem::Copy(&bookIndex, s + idx, 2);
		idx += 2;
		Mem::Copy(&totalBookRec, s + idx, 2);
		idx += 2;
		Mem::Copy(simpleName, s + idx, 8);
		idx += 8;
		Mem::Copy(complexName, s + idx, 32);
		idx += 32;

		BookInfoPtr bookInfo = &infVersionInfoPtr->bookInfo[i];
		bookInfo->bookNum = fixs(bookNum);
		bookInfo->bookIndex = fixs(bookIndex);
		bookInfo->totalBookRec = fixs(totalBookRec);
		Mem::Copy(bookInfo->simpleName, simpleName, 8);
		Mem::Copy(bookInfo->complexName, complexName, 32);
	}
}

void VersionInfo::unlockVersionPtr()
{

	User::Free(infVersionInfoPtr->bookInfo);
	infVersionInfoPtr->bookInfo = 0;
	User::Free(infVersionInfoPtr);
	infVersionInfoPtr = 0;
}

BookInfoPtr VersionInfo::lockBookPtr()
{
        BookInfoPtr bookInfo;
        BookInfoPtr resultBookInfo;

        UInt16 totalBooks;
        UInt16 i;

        lockVersionPtr();

        totalBooks = infVersionInfoPtr->totalBooks;

        bookInfo = infVersionInfoPtr->bookInfo;

        resultBookInfo = NULL;

        for (i=0; i<totalBooks; i++){
                if (bookInfo[i].bookNum == infBook){
                        resultBookInfo = &bookInfo[i];
                        i = totalBooks;
                }
        }

        return resultBookInfo;

}

void VersionInfo::unlockBookPtr()
{
        unlockVersionPtr();
}

char* VersionInfo::getVersionInfo(bool name)
{
        UInt16 versionInfoLen;
        char* versionInfo;

        lockVersionPtr();

        if (name){
#ifdef USE_LIB_C
                versionInfoLen = (UInt16) strlen(infVersionInfoPtr->versionName);
#else
		versionInfoLen = (UInt16) User::StringLength((const TUint8*)infVersionInfoPtr->versionName);
#endif
                versionInfo     = (char *)User::AllocL(versionInfoLen + 1);
                my_strcpy(versionInfo, infVersionInfoPtr->versionName);
        } else {
#ifdef USE_LIB_C
		versionInfoLen = (UInt16)strlen(infVersionInfoPtr->versionInfo);
#else
                versionInfoLen = (UInt16)User::StringLength((const TUint8*)infVersionInfoPtr->versionInfo);
#endif
                versionInfo     = (char *)User::AllocL(versionInfoLen + 1);
                my_strcpy(versionInfo, infVersionInfoPtr->versionInfo);
        }

        unlockVersionPtr();

        return versionInfo;
}

UInt16 VersionInfo::getWordRecord()
{
        UInt16 result;

        lockVersionPtr();
        result = infVersionInfoPtr->wordIndex;
        unlockVersionPtr();

        return result;
}

UInt16 VersionInfo::getWordTotal()
{
        UInt16 result;

        lockVersionPtr();
        result =infVersionInfoPtr->totalWordRec;
        unlockVersionPtr();

        return result;
}

UInt16 VersionInfo::getBookRecord()
{
        BookInfoPtr bookInfo;
        UInt16 bookRecord;

        bookInfo = lockBookPtr();
        if (bookInfo != NULL){
                bookRecord = bookInfo->bookIndex;
        } else {
                bookRecord = 0;
        }
        unlockBookPtr();

        return bookRecord;
}

UInt16 VersionInfo::getBookTotal()
{
        BookInfoPtr bookInfo;
        UInt16 bookRecord;

        bookInfo = lockBookPtr();
        bookRecord = bookInfo->totalBookRec;
        unlockBookPtr();
        return bookRecord;
}

char* VersionInfo::getBookName(bool complex)
{
        BookInfoPtr bookInfo;

        UInt16 bookNameLen;
        char* bookName;

        bookInfo = lockBookPtr();

        if (bookInfo == NULL){
                bookName = NULL;
        } else {
                if (complex){
#ifdef USE_LIB_C
                        bookNameLen = (UInt16)strlen(bookInfo->complexName);
#else
			bookNameLen = (UInt16)User::StringLength((const TUint8*)bookInfo->complexName);
#endif
                        bookName = (char *)User::AllocL(bookNameLen + 1);
                        my_strcpy(bookName, bookInfo->complexName);
                } else {
#ifdef USE_LIB_C
                        bookNameLen = (UInt16)strlen(bookInfo->simpleName);
#else
                        bookNameLen = (UInt16)User::StringLength((const TUint8*)bookInfo->simpleName);
#endif
                        bookName = (char *)User::AllocL(bookNameLen + 1);
                        my_strcpy(bookName, bookInfo->simpleName);
                }
        }
        unlockBookPtr();
        return bookName;
}

UInt16 VersionInfo::getTotalBook()
{
        UInt16 totalBooks;

        lockVersionPtr();
        totalBooks = infVersionInfoPtr->totalBooks;
        unlockVersionPtr();

        return totalBooks;
}

UInt16 VersionInfo::GetTotalVersion()
{
        return dbase->DmxGetTotalVersions();
}

char VersionInfo::GetSepChar()
{
        char result;

        lockVersionPtr();
        result = infVersionInfoPtr->sepChar;
        unlockVersionPtr();

        return result;
}

bool VersionInfo::GetRightAligned(UInt16 version)
{
        UInt8 result;

        infVersion = version;

        lockVersionPtr();
        result = infVersionInfoPtr->versionAttrib;
        unlockVersionPtr();

        return (result & infRightAligned)!=0;
}

bool VersionInfo::GetCopyProtected(UInt16 version)
{
        UInt8 result;

        infVersion = version;

        lockVersionPtr();
        result = infVersionInfoPtr->versionAttrib;
        unlockVersionPtr();

        return result & infCopyProtected;
}

bool VersionInfo::GetByteShifted(UInt16 version)
{
        UInt8 result;

        infVersion = version;

        lockVersionPtr();
        result = infVersionInfoPtr->versionAttrib;
        unlockVersionPtr();

        return !(result & infByteNotShifted);
}

char* VersionInfo::GetVersionInfo(UInt16 version)
{
        infVersion = version;
        return getVersionInfo(false);
}

char* VersionInfo::GetVersionName(UInt16 version)
{
        infVersion = version;
        return getVersionInfo(true);
}

UInt16 VersionInfo::GetTotalBook(UInt16 version)
{
        infVersion = version;

        return getTotalBook();
}

char* VersionInfo::GetSimpleBookName(UInt16 version, UInt16 book)
{
        infVersion = version;
        infBook = book;

        return getBookName(false);
}

char* VersionInfo::GetComplexBookName(UInt16 version, UInt16 book)
{
        infVersion = version;
        infBook = book;

        return getBookName(true);
}

UInt16 VersionInfo::GetBookRecordIndex(UInt16 version, UInt16 book)
{
        infVersion = version;
        infBook = book;

        return getBookRecord();
}

UInt16 VersionInfo::GetBookTotalRecord(UInt16 version, UInt16 book)
{
        infVersion = version;
        infBook = book;

        return getBookTotal();
}

UInt16 VersionInfo::GetWordRecordIndex(UInt16 version)
{
        infVersion = version;

        return getWordRecord();
}

UInt16 VersionInfo::GetWordTotalRecord(UInt16 version)
{
        infVersion = version;

        return getWordTotal();
}


UInt16 VersionInfo::GetTotalChapter(UInt16 version, UInt16 book)
{
        UInt16 bookRecord;
        UInt16* chapterInfoPtr;
        UInt16 totalChapter;

        infVersion = version;
        infBook = book;

        bookRecord = getBookRecord();
        chapterInfoPtr = (UInt16 *)dbase->DmxLockVersionRecord(infVersion, bookRecord);
        totalChapter = fixs(chapterInfoPtr[0]);
        //dbase->DmxUnlockVersionRecord(infVersion, bookRecord);

        return totalChapter;
}

UInt16 VersionInfo::GetTotalVerse(UInt16 version, UInt16 book, UInt16 chapter)
{
        UInt16 bookRecord;
        UInt16* chapterInfoPtr;
        UInt16 totalChapter;
        UInt16 totalVerse;

        infVersion = version;
        infBook = book;

        bookRecord = getBookRecord();

        chapterInfoPtr = (UInt16 *)dbase->DmxLockVersionRecord(infVersion, bookRecord);
        totalChapter = fixs(chapterInfoPtr[0]);
        if (chapter <= totalChapter){
                totalVerse = fixs(chapterInfoPtr[chapter]);
                totalVerse -= (chapter == 1) ? 0:fixs(chapterInfoPtr[chapter - 1]);
        } else {
                totalVerse = 0;
        }
        //dbase->DmxUnlockVersionRecord(infVersion, bookRecord);

        return totalVerse;
}

Database *VersionInfo::getDatabase()
{
	return this->dbase;
}
