/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/

#include "log.h"
#include "gdi.h"
#include "Viewer.h"
#include "CStringList.h"
#include  <eikenv.h>
#include <eikappui.h>
//#include <aknutils.h>
#include <baclipb.h>
#include <hal.h>
#include <charconv.h>
#include "Reader.h"
#include "VerseInfo.h"
#include "CStringList.h"
#include "greek.h"
#include "hebrew.h"
#include "big5.h"

#ifdef __SERIES60_30__
#ifndef 	SERIES_60
#define SERIES_60
#endif
#endif

#ifdef SERIES_60
#include <S60Bible.rsg>
#endif

Viewer::Viewer() : iVerseHead(NULL), iVerseTail(NULL),
	iStartChapter(1), iStartVerse(1), iStartLine(0), iBookList(NULL),
	iCurrentEncoding(KCharacterSetIdentifierIso88591),
	iLastFont(-1),
	iLastEnc(-1)	
{
   	iArea = TRect(0,0,0,0);
    iFont = CEikonEnv::Static()->DenseFont();
    
	iNormalWidth = 0;
	iFirstVerseWidth = 0;
	
	/*Default colors*/
	iColors[EVerseText] = KRgbBlack;
	iColors[EVerseBackground]=KRgbWhite;
	iColors[EVerseNumber] = KRgbBlue;
	iColors[EHeaderText] = KRgbBlue;
	iColors[EHeaderFrame] = KRgbBlue;
	/*Blue*/
	iColors[EHeaderBG] = TRgb(206, 206, 255);		

	/*Default 100% Zoom*/
	iZoomLevel = 2;
	/*Use long name*/
	iLongName = 1;
	
	/*Zoom font size in twips*/
	zoom[0] = 75;
	zoom[1] = 100;
	zoom[2] = 150;
	zoom[3] = 175;
	zoom[4] = 200;	
	
}

/*destructor*/
Viewer::~Viewer()
{
	if (iCustomFont) CEikonEnv::Static()->ReleaseScreenFont(iCustomFont);
	ClearVerse();	
	delete iBookList;
	delete iBookmarkList;
	delete iReader;
	delete iCharacterConverter;
	iFs.Close();
}

/*Set color for a text object*/
void Viewer::SetColor(ETextObject aTextObject, TRgb aColor)
{
	iColors[aTextObject] = aColor;
}

/*Get color for a text object*/
TRgb Viewer::GetColor(ETextObject aTextObject) const
{
	return iColors[aTextObject];
}

void Viewer::CopyToClipboardL(int count)
{
	RFs fsSession;
	fsSession.Connect();
	CClipboard* cb = CClipboard::NewForWritingLC(fsSession);
	CPlainText* plaintext = CPlainText::NewL();

	CleanupStack::PushL(plaintext);

	VerseInfo *v = GetVerseHead();

	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}

        //this should not happen
        if (!v) return;

	TBuf<120> strversenum;
	char *s = BookName();
	TBuf<256> stat;
	MyStringToDes(s, stat);
	User::Free(s);
	//designed to be able to copy more than one verse
	//but can't be done from the UI yet
	for (int i = 0; i<count; i++){
		for (int j = 0; j<v->lines->Count(); j++) {
			if (j== 0) {
				strversenum.Zero();
				_LIT(CopyFormat,"%S %d:%d ");
				strversenum.Format(CopyFormat, &stat, v->chapter,
					v->verse);
				plaintext->InsertL(plaintext->DocumentLength(),
					strversenum);
			}
			plaintext->InsertL(plaintext->DocumentLength(), _L(" "));
			plaintext->InsertL(plaintext->DocumentLength(), v->lines->At(j));
		}
		v = v->next;
		if (!v) break;
	}

	plaintext->CopyToStoreL(cb->Store(), cb->StreamDictionary(), 0,
		plaintext->DocumentLength());
	cb->CommitL();
	CleanupStack::PopAndDestroy(plaintext);
	CleanupStack::PopAndDestroy(cb);
	fsSession.Close();
}

void Viewer::ConstructL(RFs &aFs)
{
	iFs = aFs;
	iReader = NULL;
    iBookmarkList = CStringList::NewL(CEikonEnv::Static()->FsSession());
	_LIT(EMPTY, "");
	iCurrentBook = EMPTY;
    iFs.Connect();
	iCharacterConverter = CCnvCharacterSetConverter::NewL();
    iConverterAvailability = iCharacterConverter->PrepareToConvertToOrFromL(iCurrentEncoding, iFs);
	isGreek = EFalse;
	isHebrew = EFalse;
	isBIG5  = EFalse;
}


void Viewer::AddToBookmark()
{
	VerseInfo *v = GetVerseHead();

	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}

	TBuf<120> strversenum;
	TBuf<200> bookmarkLine;
	char *s = BookName();
	TBuf<256> stat;
	MyStringToDes(s, stat);
	User::Free(s);
	if (v->linecount==0) return;
	strversenum.Zero();
	_LIT(KFormat,"\t%S %d:%d\t ");
	strversenum.Format(KFormat, &stat, v->chapter, v->verse);
	bookmarkLine = strversenum;
	bookmarkLine.Append(v->lines->At(0));
	iBookmarkList->AddStringL(bookmarkLine);
}

void Viewer::LoadBookmarkL()
{
	iBookmarkList->LoadFromFileL(iBookmarkFile);
}

void Viewer::SaveBookmarkL()
{
	iBookmarkList->SaveToFileL(iBookmarkFile);
}

void Viewer::DelBookmark(int idx)
{
	iBookmarkList->Content()->Delete(idx);
}

void Viewer::DelAllBookmark()
{
	iBookmarkList->Content()->Reset();
}

CDesCArray* Viewer::GetBookmark()
{
	return iBookmarkList->Content();
}

int Viewer::GetTotalBookChapter(int book)
{
	return iReader->GetTotalBookChapter(book);
	
}

int Viewer::GetTotalChapterVerse(int book, int chapter)
{
		return iReader->GetTotalChapterVerse(book, chapter);	
}


const char *lookup_name(int booknumber)
{
	/*probabilistic search*/
	int i = booknumber/10;
	
	if (i>booknames_size) {
		i = booknames_size-1;
	}
	if (englishbooknames[i].number>booknumber) {
		while (i>=0) {
			if (englishbooknames[i].number==booknumber) {
				return englishbooknames[i].name;
			}
			if (englishbooknames[i].number<booknumber) {
				return NULL;
			}			
			i--;
		}	
	} else {
		while (i<booknames_size) {
			if (englishbooknames[i].number==booknumber) {
				return englishbooknames[i].name;
			}
			if (englishbooknames[i].number>booknumber) {
				return NULL;
			}			
			i++;
		}	
	}
	
	return NULL;
}

void Viewer::FillBookList()
{
	if (!IsReady()) return;
	if (iBookList) {
		delete iBookList;
		iBookList = 0;
	}
	iBookList = new (ELeave) CDesCArrayFlat(4);
	int booknum = iReader->GetFirstBook();
	int idx = 0;
	do {
		iBookNumberList[idx++] = booknum;
		int chapter_count, verse_count;
		char *tbookname = iReader->GetBookInfo(booknum, chapter_count, verse_count);
		TBuf<128> bookname;
		MyStringToDes(tbookname, bookname);
		User::Free(tbookname);

#ifdef SERIES_60		
		TBuf<256> bookinfo;
		/*
		if (chapter_count>1) {
                      HBufC* format = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_FORMAT_CHAPTER_PLURAL);
			 //_LIT(KFormat,"\t%S\t%d Chapters");
                      bookinfo.Format(*format, &bookname, chapter_count);
                      CleanupStack::PopAndDestroy(format);
		} else {
                        HBufC* format = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_FORMAT_CHAPTER);
			//_LIT(KFormat,"\t%S\t1 Chapter");
			bookinfo.Format(*format, &bookname);
                        CleanupStack::PopAndDestroy(format);
		}*/

		const char *engname = lookup_name(booknum);
		if (engname!=NULL) {		
			_LIT(KFormat, "\t%S\t%S\t");
			TBuf<128> engbookname;
			StringToDes(engname, engbookname);			
			bookinfo.Format(KFormat, &bookname, &engbookname);
		} else {
			_LIT(KFormat2, "\t%S\t\t");	
			bookinfo.Format(KFormat2, &bookname);
		}
		
		iBookList->AppendL(bookinfo);
#else		
		TBuf<256> bookinfo;
		const char *engname = lookup_name(booknum);
		if (engname!=NULL) {		
			_LIT(KFormat, "%S (%S)");
			TBuf<128> engbookname;
			StringToDes(engname, engbookname);			
			bookinfo.Format(KFormat, &bookname, &engbookname);
		} else {
			_LIT(KFormat2, "%S");	
			bookinfo.Format(KFormat2, &bookname);
		}

		//iBookList->AppendL(bookname);
		iBookList->AppendL(bookinfo);

#endif
		booknum = iReader->GetNextBook(booknum);

	} while (booknum!=-1);
}

int Viewer::GetGlobal(int index)
{
	return iGlobal[index];	
}

void Viewer::SetGlobal(int index, int value)
{
	iGlobal[index] = value;	
}


CDesCArray* Viewer::GetBookList()
{
	if (!iBookList) FillBookList();
	return iBookList;
}

void Viewer::ClearBookList()
{
	if (iBookList) {
		delete iBookList;
		iBookList = NULL;
	}
}


void Viewer::CalcScrollLen()
{
	iScrollLen = ((iArea.Height()-20-GetFont()->HeightInPixels()) /
		(GetFont()->HeightInPixels()));
}

void Viewer::SetArea(const TRect &aRect)
{
	iArea = aRect;
	CalcScrollLen();
	FillVerse();
}

TBool Viewer::ChangeBibleL(const TDesC &filename)
{
	int tmp_b = -1, tmp_s=1, tmp_c=1;
	//save current position for the new bible
	//Log(_L("ChangeBibleL"));
	if (iReader) {
		Log(_L("Delete Old IReader"));
		tmp_b = iReader->BookNumber();
		tmp_s = iStartVerse;
		tmp_c = iStartChapter;
		ClearVerse();
		delete iReader;
		iReader = NULL;
	}
	//Log(_L("Creating IReader"));
	
	
	iReader = new (ELeave) Reader();
	Log(_L("Construct"));
	
	TRAPD(error, iReader->ConstructL(iFs, filename));
	Log(_L("Check Err"));
   	if (error!=KErrNone) {
   		delete iReader;
   		iReader = NULL;
   		return EFalse;
   	}
	//Log(_L("OK"));
	
   	//FillBookList();
    	iCurrentBook = filename;
   	if (tmp_b!=-1) {
   		if (!iReader->SeekVerse(1, (UInt16)tmp_b,
				(UInt16) tmp_c,(UInt16) tmp_s))	{
			iReader->SeekVerse(1, (UInt16)iReader->GetFirstBook(), 1, 1);
   		}
   	} else {
   		iReader->SeekVerse(1, (UInt16)iReader->GetFirstBook(), 1, 1);
   	}
	TParse p;
	p.SetNoWild(filename, NULL, NULL);
	TPtrC first = p.Name().Left(1);
	isHebrew = EFalse;
	isGreek = EFalse;
	isBIG5 = EFalse;
	if (first==_L("z") || first==_L("Z")) {
		isGreek = ETrue;		
	} else if (first==_L("q") || first==_L("Q")) {
			isHebrew = ETrue;
			Log(_L("Hebrew "));
	}  else {
		if (p.Name().FindF(_L("big5"))!=KErrNotFound) {
			isBIG5  = ETrue;
		}
	}
    iStartChapter = iReader->ChapterNumber();
    iStartVerse = iReader->VerseNumber();
	Log(_L("Done"));

   	return ETrue;
}

TBool Viewer::ChangeBible(const TDesC &filename)
{
	ChangeBibleL(filename);
	Log(_L("ChangeBible"));	
	return (iReader!=NULL);
}

void Viewer::Init()
{
	iFont = CEikonEnv::Static()->DenseFont();	
	iArea = CEikonEnv::Static()->EikAppUi()->ClientRect();
	CalcScrollLen();
}

int Viewer::GetFontNum()
{
	return iFontNum;
}

void Viewer::setFont(int num)
{
	CWsScreenDevice* dev = CEikonEnv::Static()->ScreenDevice();
	TTypefaceSupport typefaceSupport;
	dev->TypefaceSupport(typefaceSupport, num);
	TInt fontSize = zoom[iZoomLevel];
	TFontSpec fontSpec(typefaceSupport.iTypeface.iName, fontSize);
	iFontNum = num;
	if (iCustomFont) {
		CEikonEnv::Static()->ReleaseScreenFont(iCustomFont);	
	}
#ifdef __SERIES60_30__	
	iCustomFont = CEikonEnv::Static()->CreateScreenFontL(fontSpec);
#else
	CCoeEnv::Static()->ScreenDevice()->GetNearestFontInTwips(iCustomFont, fontSpec);
#endif
	CalcScrollLen();
	if (iLastFont==-1) {
		iLastFont = iFontNum;
		return;
	}
	if (iLastFont!=iFontNum) { 
		ClearVerseList();
		FillVerse();
		iLastFont = iFontNum;
	}
}

const CFont *Viewer::GetFont()
{
	if (iCustomFont==NULL) {
#ifdef __SERIES60_30__	
	_LIT(FontName, "Nokia Sans S60");
	TInt fontSize = 150;
#else
	_LIT(FontName, "LatinBold12");
	TInt fontSize = 75;
#endif	

		TFontSpec fontSpec(FontName, fontSize);
		iCustomFont = CEikonEnv::Static()->CreateScreenFontL(fontSpec);
	}
	return(iCustomFont==NULL)?iFont:iCustomFont;
}

/*Get verse list head*/
VerseInfo* Viewer::GetVerseHead()
{
	return this->iVerseHead;
}

/*Add as the last element of verse list*/
void Viewer::AddLast(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		iVerseHead = v;
		iVerseTail = v;
	} else {
		iVerseTail->next = v;
		v->prev = iVerseTail;
		iVerseTail = v;
	}
	iVerseTail->next = NULL;
}

/*Add as first element of verse list*/
void Viewer::AddFirst(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		iVerseHead = v;
		iVerseTail = v;
	} else {
		v->next = iVerseHead;
		iVerseHead->prev = v;
		iVerseHead = v;
	}
	iVerseHead->prev = NULL;
}


/*Check wheter a verse already exists on verse list*/
int Viewer::VerseExisted(int chapter, int verse)
{
	VerseInfo *tmp = iVerseHead;
	while (tmp!=NULL) {
		if (tmp->chapter==chapter) {
                  if (tmp->verse==verse) return 1;
                  if (tmp->verse>verse) return 0;
		}
        if (tmp->chapter>chapter) return 0;
        tmp = tmp->next;
	}
	return 0;
}

/**
 * Add at a correct position
 */
void Viewer::Add(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		AddFirst(v);
		return;
	}
	VerseInfo *tmp = iVerseHead;
        //add first
        if (v->chapter<iVerseHead->chapter) {
            AddFirst(v);
            return;
        }
        if (v->chapter==iVerseHead->chapter && v->verse<iVerseHead->verse) {
            AddFirst(v);
            return;
        }
        //add last
        if (v->chapter>iVerseTail->chapter) {
            AddLast(v);
            return;
        }
        if (v->chapter==iVerseTail->chapter && v->verse>iVerseTail->verse) {
            AddLast(v);
            return;
        }
        //add in the middle, list is not empty
	while (tmp!=NULL) {
            if (v->chapter<tmp->chapter || (v->chapter==tmp->chapter && v->verse<tmp->verse)) {
                //insert before tmp
                v->prev = tmp->prev;
                tmp->prev->next = v;
                v->next = tmp;
                tmp->prev = v;
                break;
            }
            tmp = tmp->next;
	}
}


/*Delete first verse on the verse list*/
void Viewer::DelFirst()
{
	VerseInfo *tmp;
	if (iVerseHead!=NULL) {
		tmp = iVerseHead;
		iVerseHead=iVerseHead->next;
		if (iVerseHead==NULL) {
			iVerseTail=NULL;
		} else {
			iVerseHead->prev = NULL;
		}
		delete tmp;
	}
}

/*Delete last verse on the verse list*/
void Viewer::DelLast()
{
	VerseInfo *tmp, *prev;
	if (iVerseHead==NULL) return;
	tmp = iVerseTail;
	prev = iVerseTail->prev;

	if (prev==NULL) {
		delete iVerseHead;
		iVerseHead = NULL;
		iVerseTail = NULL;
	} else {
		iVerseTail = prev;
		iVerseTail->next = NULL;
		delete tmp;
	}
}

/*Wrap a verse to multiple line, and format the line
* For hebrew text, also reverses the character order.
*/
void Viewer::WrapVerse  (VerseInfo *v) 
{
	int i = 0, rest, c;
	int pos;
	CFont::TMeasureTextInput input;
	CFont::TMeasureTextOutput output;
	iLinePos[iLineCount] = (UInt16) User::StringLength((const TUint8*)iTmpBuff);
	//rest = aStringToWrap.Length();
	char tmpdel = 0;
    v->lines = new (ELeave)CArrayFixFlat<TPtrC>(30);
    v->lines->Reset();
    v->lineformat = new (ELeave)CArrayFixFlat<TInt>(30);
    v->lineformat->Reset();
	int first = 1;
	int gotverse = 0;
	v->vcount = iLineCount;
	v->vcontent = (HBufC**)User::AllocL(sizeof(HBufC*)*iLineCount);
	
	
	for (int line = 0; line<iLineCount; line++){
		tmpdel = iTmpBuff[iLinePos[line+1]];
		/*mark end of string*/
		iTmpBuff[iLinePos[line+1]] ='\0';
		/*convert string to unicode*/
		if (iLineFormat[line]==Decoder::versTextType) {
			gotverse = 1;
		}
		if (iLineFormat[line]==Decoder::chapTextType && (isGreek || isHebrew)) {
			MyStringToDes(iTmpBuff+iLinePos[line], iVerseContent);
		} else {
				if (isGreek) {
					GreekStringToDes(iTmpBuff+iLinePos[line], iVerseContent);
				} else if (isHebrew) {
					HebrewStringToDes(iTmpBuff+iLinePos[line], iVerseContent);
				} else if (isBIG5) {
					Big5StringToDes(iTmpBuff+iLinePos[line], iVerseContent);
				} else {
					MyStringToDes(iTmpBuff+iLinePos[line], iVerseContent);
				}			
		}
		/*unmark end of string*/
		iTmpBuff[iLinePos[line+1]] = tmpdel;
			
		rest = iVerseContent.Length();
		v->vcontent[line] = iVerseContent.AllocL();		
		TDesC &text = *(v->vcontent[line]);
		pos = 0;
		int sepChar = iReader->GetSepChar();
		const CFont *font = GetFont();
		while (rest>0) {
			if (first && gotverse) {
				input.iMaxBounds = iFirstVerseWidth;
				v->verseline = v->lines->Count();
				first = 0;
			} else {
				input.iMaxBounds = iNormalWidth;
			}
			font->MeasureText(text.Mid(pos), &input, &output);		
			c = output.iChars;
			if (sepChar!=0) {
				if (rest - c>0) {
					if (text[c+pos]!=sepChar) {
						while (c>0 && text[c+pos]!=sepChar) c--;						
					}
				}
			}
			if (c==0) c=1;
			v->lines->AppendL(text.Mid(pos, c));
			v->lineformat->AppendL(iLineFormat[line]);
			if (isHebrew && iLineFormat[line]==Decoder::versTextType) {
					TPtrC s = (v->lines)->At(v->lines->Count()-1);
					TUint16* st = const_cast<TUint16*>(s.Ptr());
					for (int k = 0; k<s.Length()/2; k++) {
							TUint16 tmp = st[k];
							st[k] = st[s.Length()-k-1];
							st[s.Length()-k-1] = tmp;
					}
				
			}
			
			
			v->linecount++;
			rest -= c;
			pos += c;
			i++;
		}
		if (iLineFormat[line]==Decoder::bookTextType) {
			v->lines->AppendL(_L(""));
			v->lineformat->AppendL(Decoder::versTextType);
		}
	}
}

/*
* Returns 0 OK, 1 up, 2 down
*/
int  Viewer::IsScreenCovered()
{
	VerseInfo *v = iVerseHead;
	int line_count =0;
	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		line_count += v->linecount;
		v = v->next;
	}
	if (!v) return EFalse;	
	/*we must have at least a previous verse*/
	if (!v->prev) {
		if (!(v->chapter==1 &&  v->chapter==1)) {
			return 1;
		} else {
			if (!(iVerseHead && (iVerseHead->chapter==1) && (iVerseHead->verse==1))){
				if (line_count<iScrollLen) return 1;
			}
		}
	}
	/*we must have at least a next verse*/
	if (!v->next) return 2;
	/*we must not count the lines above the screen*/
	line_count = -iStartLine;
	/*count the number of lines remaining*/
	while (v) {
		line_count += v->linecount;
		v = v->next;
	}
	return (line_count>=iScrollLen)?0:2;
}

#define MAX_VERSE_IN_MEM 50
#define VERSE_LIMIT (MAX_VERSE_IN_MEM/2)


int Viewer::AddVerse()
{

	//Log(_L("check verse"));
	if (VerseExisted(iReader->ChapterNumber(), iReader->VerseNumber())) {
		return iStartVerse;
	}
	/*this is needed for the line calculation*/
    TRect drawRect=iArea;
		
	if (iArea.IsEmpty()) {
        drawRect=CEikonEnv::Static()->EikAppUi()->ClientRect();
    }
	/*normal line width (line without verse number)*/
	iNormalWidth = (TInt)drawRect.Width()-5;

	TBuf<8> strversenum;
	//Log(_L("create vinfo"));
    VerseInfo *v = new (ELeave)VerseInfo(iReader->ChapterNumber(), iReader->VerseNumber());
    strversenum.Zero();
    strversenum.AppendNum(iReader->VerseNumber());
    int sw = GetFont()->TextWidthInPixels(strversenum)+5;
	if (sw==5) { /*TextWidthInPixels returns 0*/
		sw = CEikonEnv::Static()->NormalFont()->TextWidthInPixels(strversenum)+5;
	}
	iTmpBuff[0]='\0';
	iLineCount = 0;
	//Log(_L("Get current str"));
    if (!iReader->GetCurrentVerseAsStringF(iTmpBuff, 
	iLinePos, iLineFormat, &iLineCount)) {
		delete v;
		return -1;
	}
	iFirstVerseWidth = iNormalWidth-sw-2;
	//Log(_L("wrapp"));
    WrapVerse(v);
    //Log(_L("add"));
	Add(v);
	return v->verse;
}


void Viewer::TrimDown()
{
	VerseInfo *v = iVerseHead;
	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	if (!v) return;	
	int line_count = -iStartLine;
	/*count the number of lines remaining*/
	while (v) {
		line_count += v->linecount;
		v = v->next;
		if (line_count>iScrollLen*2) break;
	}
	if (v!=NULL) {
		int delv = 0;
		while (v) {
			delv++;
			v = v->next;
		};
		for (int i=0; i<delv; i++){
			DelLast();
		}
	}
	
}

void Viewer::TrimUp()
{
	VerseInfo *v = iVerseHead;
	int line_count = 0;
	int del_count = 0;
	while (v) {
		del_count++;
		line_count += v->linecount;
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	if (line_count<iScrollLen*2) return;
	if (del_count>3) {
		for (int i=0; i<del_count-2; i++){
			DelFirst();
		}
	}
}


/* The most important data structure for this viewer is a double linked list with element type of VerseInfo
  * The list look like this:
  * NULL <-- -verse1 <---> verse2 <---> verse3 ----> NULL
  * There is also iVerseHead that points to the first verse (verse1), and iVerseTail that points to the last verse.
  *
  * The currently displayed verse in screen is the one that has verse==iStartVerse and chapter==iStartChapter starting from line iStartLine.
  * It is up to the displayer to display as much verse as possible starting from that point.
  *
  * The list is initially small and will grow as much as needed. The list will not grow unlimited. If we go to a different book or jumping from chapters, 
  *  then the verse list is cleared.
  *
  * If we scroll up or down, the list will be updated (add to the first or last element of the list)
 */
void Viewer::FillVerse()
{
	

	if (!IsReady()) return;
	//Log(_L("Start Fill Verse"));
    
	/*if  iStartChapter is undefined, we ask it from the iReader*/
	if (iStartChapter == -1) {
		iStartChapter = iReader->ChapterNumber();
        iStartVerse = iReader->VerseNumber();
    }
    //LogL(_L("Seeking chapter"), iStartChapter);
	//LogL(_L("Seeking verse"), iStartVerse);    
	/*Seek to the current chapter and verse*/
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	
	/*check if the verse we are looking for exists on the list, if not, we must add it to the list*/
	if (!VerseExisted(iStartChapter, iStartVerse)) {
		//Log(_L("Adding current verse"));
        AddVerse();		
	}
	//Log(_L("Starting to cover "));
	int lastverse = iStartVerse;
	int cover = IsScreenCovered();
	if (cover==0) {
		//Log(_L("Done, covered"));
		return;
	}
	if (cover==1) { /*previous is not covered*/		
		//Log(_L("Covering prev"));
		while (cover!=0) {		
			if (cover==2) break;
			iReader->MovePrevVerse();		
			AddVerse();
			if (iReader->ChapterNumber()==1 && iReader->VerseNumber()==1) {
				break;
			}
			cover = IsScreenCovered();
		}
		TrimDown();
	}
	
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	
	cover = IsScreenCovered();
	while (cover!=0) {		
		if (cover==1) break; /*this should not happen*/
		iReader->MoveNextVerse();
		if (lastverse==iReader->VerseNumber()) {
			break;
		}
		lastverse = AddVerse();
		if (lastverse==-1) break;
		cover = IsScreenCovered();
	}
	TrimUp();
	//Log(_L("Done"));	
}

/*Clear verse list, and reset current verse, chapter, and line*/
void Viewer::ClearVerse()
{
	while (iVerseHead) DelFirst();
	iStartVerse = -1;
	iStartChapter = -1;
	iStartLine = 0;
}

/*Clear verse list, without reseting current verse, chapter, and line*/
void Viewer::ClearVerseList()
{
	while (iVerseHead) DelFirst();
}


/*Update a buffer if we almost reached end of verse list*/
void Viewer::UpdateBuffer()
{
	VerseInfo *v = iVerseHead;
	int line_count = 0;
	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	/*we must not count the lines above the screen*/
	line_count = -iStartLine;
	/*count the number of lines remaining*/
	while (v) {
		line_count += v->linecount;
		v = v->next;
	}
	if (line_count < (iScrollLen)) FillVerse();
}


/*Temporaryly save current state (not stack based, can only be done once)*/
void Viewer::SaveState()
{
	tmpStartVerse = iStartVerse;
	tmpStartChapter = iStartChapter;
}

/*Load currently saved state*/
void Viewer::LoadState()
{
	iStartVerse = tmpStartVerse;
	iStartChapter = tmpStartChapter;
}

/* Scrollup: scroll up count lines
 * This method is a bit complicated since a verse can have multiple lines, and some of the lines may not yet in the memory
 * The fist displayed  line is denoted by iStartChapter, iStartVerse, and iStartLine
 * First we try to decrease the iStartLine, if that doesn't work we try the previous line
 * if we run out previous line, we must load it again using fillverse
 */

void Viewer::ScrollUp(int count)
{
	if (!IsReady()) return;
	VerseInfo *v = iVerseHead;

restart_down_search:
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	//scroll in one verse
	if (count<=iStartLine) {
		iStartLine -= count;
		return;
	}

	v = v->prev;
	if (!v) { //start of buffer reached
		iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
			(UInt16)iStartVerse);
		iReader->MovePrevVerse();
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		while (v) {
			if (v->verse==iStartVerse && v->chapter==iStartChapter) {
				break;
			}
			v = v->next;
		}
		v = v->prev;
	}
	count -= iStartLine;
	iStartLine = 0;
	if (!v) return; //useless, start of book reached
	while (v && count>0) {
		iStartVerse = v->verse;
		iStartChapter = v->chapter;
		if (v->linecount>=count) {
			iStartLine = v->linecount-count;
			count = 0;
			break;
		} else {
			count -= v->linecount;
		}
		v = v->prev;
		if (v && count == 0) {
			iStartVerse = v->verse;
			iStartChapter = v->chapter;
		}
	}
	if (count>0) { //long scroll
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		goto restart_down_search;
	}
}

/*this has the similar logic like ScrollUp*/
void Viewer::ScrollDown(int count)
{
	if (!IsReady()) return;
	VerseInfo *v = iVerseHead;

restart_up_search:
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}

	//scroll in one verse
	if (count<v->linecount-iStartLine) {
		iStartLine+=count;
		return;
	}

	count -= (v->linecount-iStartLine);


	iStartLine = 0;

	v=v->next;

	if (!v) { //end of buffer reached
		iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
			(UInt16)iStartVerse);
		iReader->MoveNextVerse();
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		while (v) {
			if (v->verse==iStartVerse && v->chapter==iStartChapter) {
				break;
			}
			v = v->next;
		}
		v = v->next;
	}

	if (!v) return; //useless, end of book reached
	iStartVerse = v->verse;
	iStartChapter = v->chapter;

	while (v && count>0) {
		iStartVerse = v->verse;
		iStartChapter = v->chapter;

		if (v->linecount>count) {
			iStartLine = count;
			count = 0;
			break;
		} else {
			count -= v->linecount;
		}
		v = v->next;
		if (v && count == 0) {
			iStartVerse = v->verse;
			iStartChapter = v->chapter;
		}

	}
	if (count>0) { //long scroll
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		goto restart_up_search;
	}
}



/*Go to next verse*/
void Viewer::NextVerse()
{
	/*Do nothing if no bible is loaded*/
	if (!IsReady()) return;

	VerseInfo *v = iVerseHead;

	/*find the current verse in the list*/
	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	
	/*if current verse is found, and we have the next verse*/
	if (v && v->next) {
		v = v->next;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		iStartLine = 0;
		return;
	}
	//we need to refill the verse list by going to the current verse, and ask the reader to go to next verse
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter, (UInt16)iStartVerse);
	iReader->MoveNextVerse();
	FillVerse(); //refill buffer
	v = iVerseHead; //restart search
	
	/*The fillverse method will refill some previous verses also, so we must search again*/
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	/*if we found it, then go to that verse*/
	if (v && v->next) {
		v = v->next;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		return;
	}

}

/*Go to previous verse*/
void Viewer::PrevVerse()
{
	/*Do nothing if no bible is loaded*/
	if (!IsReady()) return;

	VerseInfo *v = iVerseHead;

	/*find the current verse in the list*/
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	/*if current verse is found, and we have the previus verse*/
	if (v && v->prev) {
		v = v->prev; /*Set current display to the previous verse*/
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		iStartLine = 0;
		return;
	}
	
	/*if we are already on the first verse of the first chapter, then do nothing*/
	if (v && v->chapter==1 && v->verse==1) {
		iStartLine = 0;
		return;
	}
	
	//we need to refill the verse list by going to the current verse, and ask the reader to go to previus verse
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter, (UInt16)iStartVerse);
	iReader->MovePrevVerse();
	FillVerse(); //refill buffer
	v = iVerseHead; //restart search
	/*The fillverse method will refill some previous verses also, so we must search again*/
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	/*if we found it, then go to that verse*/
	if (v && v->prev) {
		v = v->prev;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		iStartLine = 0;
		return;
	}
}


/*Go to the first verse in a book*/
void Viewer::GotoTop()
{
	iStartChapter = 1;
	iStartVerse = 1;
	iStartLine = 0;
	UpdateBuffer();
}

/*Go to the last verse in a book*/
void Viewer::GotoBottom()
{
	if (!IsReady()) return ;
	iReader->GetLastVerse(&iStartChapter,  &iStartVerse);
	iStartLine = 0;
	UpdateBuffer();
}

/*Goto next chapter, nothing is done if no bible is loaded or we are on the last chapter*/
void Viewer::NextChapter()
{
	if (!IsReady()) return ;
        int oldChapter = iStartChapter;
	iStartChapter++;

	if (iStartChapter>iReader->getTotalChapter()) {
		iStartChapter = iReader->getTotalChapter();
	}
    if (oldChapter==iStartChapter) return;
    ClearVerseList();
    iStartVerse = 1;
	iStartLine = 0;
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	FillVerse();
}

/*Goto previous chapter, nothing is done if no bible is loaded or we are on the first chapter*/
void Viewer::PrevChapter()
{
	if (!IsReady()) return ;
    if (iStartChapter==1) {
        return;
    }
	int tmp = iStartChapter;
    ClearVerseList();
	iStartChapter = tmp;
	iStartChapter--;
	iStartVerse = 1;
	iStartLine = 0;
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	FillVerse();
}

/*go to previous page*/
void Viewer::PageUp()
{
	ScrollUp(iScrollLen);
	UpdateBuffer();
}

/*go to next page*/
void Viewer::PageDown()
{
	ScrollDown(iScrollLen);
	UpdateBuffer();
}


/*returns  complete  book name */
char *Viewer::BookName()
{
	return iReader->BookName();
}

/*returns simple (short)  book name (e.g: Gen for Genesis)*/
char* Viewer::SimpleBookName()
{
	return iReader->SimpleBookName();
}

/*seek a verse using bible version, book number, chapter and verse */
bool Viewer::SeekVerse(UInt16 version, UInt16 book, UInt16
				chapter, UInt16 verse)
{
	if (!IsReady()) return false;
	return iReader->SeekVerse(version, book, chapter, verse);
}

/*seek a verse using string, e.g "John 3:16" */
bool Viewer::SeekVerseAsString(UInt16 version, char *str)
{
	if (!IsReady()) return false;
	return iReader->SeekVerseAsString(version, str);
}

/*Goto to first verse of the first chapter on the next book, nothing is done if no bible is loaded or if we are on the last book*/
void Viewer::NextBook()
{
	if (!IsReady()) return;
	if (iReader->GetNextBook()!=-1) {
		ClearVerse();
		SeekVerse(1, (UInt16)iReader->GetNextBook(), 1, 1);
		FillVerse();
	}
}

/*Goto to first verse of the first chapter on the previous book, nothing is done if no bible is loaded or if we are on the first book*/
void Viewer::PrevBook()
{
	if (!IsReady()) return;
	if (iReader->GetPrevBook()!=-1) {
		iStartChapter = 1;
		iStartVerse = 1;
		iStartLine = 0;
		ClearVerse();
		SeekVerse(1, (UInt16)iReader->GetPrevBook(), 1, 1);
		FillVerse();
	}
}


/*Return currently displayed chapter on the first line of the screen*/
int Viewer::GetStartChapter()
{
	return iStartChapter;
}

/*Return currently displayed verse on the first line of the screen*/
int Viewer::GetStartVerse()
{
	return iStartVerse;
}

/*Return currently displayed line of a verse on the first line of the screen*/
int Viewer::GetStartLine()
{
	return iStartLine;
}

/*returns the current book number*/
int Viewer::GetBookNumber()
{
    if (!IsReady()) return 0;
    return iReader->BookNumber();
}



/*Returns wheter a bible is loaded*/
int Viewer::IsReady()
{
	return iReader!=NULL;
}

/*Go to a book by number. Number is specified on the PDB file*/
void Viewer::GotoBook(int book)
{
	if (!IsReady()) return;
	iReader->SeekVerse(1, (UInt16)book, 1, 1);
	ClearVerse();
	FillVerse();
}

/*Go to a book by index (used for selecting booklist)*/
void Viewer::GotoBookByIndex(int idx)
{
	if (!IsReady()) return;
	iReader->SeekVerse(1, (UInt16)iBookNumberList[idx], 1, 1);
	ClearVerse();
	FillVerse();
}

int Viewer::GetBookIndex()
{
    if (!IsReady()) return 0;
    if (!iBookList) return 0;
    int num = iReader->BookNumber();
    int l = iBookList->Count();
    for (int i = 0; i<l; i++){
    	if 	(iBookNumberList[i]==num){
    		return i;
    	}
    }
    return 0;
}


/*get binding at specific index*/
TInt& Viewer::GetBinding(int idx)
{
	return iBinding[idx];
}

/*Load bindings from file*/
void Viewer::LoadBindings()
{
    int i;
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBindingsFile);
	Mem::FillZ(iBinding, sizeof(iBinding));
	for (i = 0; i<settings->Count();i++){
		iBinding[i] =settings->GetNumberAt(i);
	}
	CleanupStack::PopAndDestroy(settings);
}

/*Save bindings to file*/
void Viewer::SaveBindings()
{
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	for (int i  = 0; i<15; i++) {
		settings->AddNumberL(iBinding[i]);
	}
	settings->SaveToFileL(iBindingsFile);
	CleanupStack::PopAndDestroy(settings);
}

/* Converts a C string to a descriptor
 * taken from the putty source code
 */
void Viewer::StringToDes(const char *aStr, TDes &aTarget) {

    // Doesn't really work for anything except US-ASCII
    // Copied from putty
    // Used as fallback if characterconverter is not available
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(*aStr++);
    }
}

/*set encoding, and update character converter*/
void Viewer::SetEncoding(TUint Encoding) {
	iCurrentEncoding = Encoding;
	//fist time
	if (iLastEnc==-1) {
		iLastEnc = iCurrentEncoding;
		return;
	}
	if ((TUint)iLastEnc!=iCurrentEncoding) {
		delete iCharacterConverter;
		iCharacterConverter = CCnvCharacterSetConverter::NewLC();
		iConverterAvailability = iCharacterConverter->PrepareToConvertToOrFromL(iCurrentEncoding, iFs);
		CleanupStack::Pop(iCharacterConverter);
		ClearVerse();
		FillVerse();
		iLastEnc = iCurrentEncoding;
	}
}

/*return the current encoding*/
TUint Viewer::GetEncoding()
{
	return iCurrentEncoding;
}

/* Converts a descriptor into a C string. The string needs to be pre-allocated
 * taken from the putty source code
 */
void Viewer::DesToString(const TDesC &aDes, char *aTarget) {
    int i = 0;
    int len = aDes.Length();
    while ( i < len ) {
        *aTarget = (char) aDes[i];
        aTarget++;
        i++;
    }
    *aTarget = 0;
}


/*Convert from String To DES
 If the current bible is BIG5, then Big5StringToDes() will be called*/
void Viewer::MyStringToDes(const char *aStr, TDes &aTarget)
{
	if (isBIG5) {
		Big5StringToDes(aStr, aTarget);
		return;
	}
      TInt aState = CCnvCharacterSetConverter::KStateDefault;
      if (iConverterAvailability == CCnvCharacterSetConverter::EAvailable) {
	TPtrC8 tmp((TUint8*)aStr, User::StringLength((const TUint8*)aStr));
        iCharacterConverter->ConvertToUnicode(aTarget, tmp, aState);
      } else {
          StringToDes(aStr, aTarget);
      }
}

/*The greek encoding in PDB file is not available  in Symbian.
   This is just a simple lookup*/

void Viewer::GreekStringToDes(const char *aStr, TDes &aTarget)
{
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(greektab[(int)*aStr]);
		aStr++;
    }
}


/*The hebrew encoding in PDB file is not available  in Symbian.
This is just a simple lookup*/
void Viewer::HebrewStringToDes(const char *aStr, TDes &aTarget)
{
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(hebrewtab[(int)*aStr]);
		aStr++;
    }
}


/*Convert from BIG5 encoding to UCS-2 taken from MBFL library*/

#define MBFL_WCSGROUP_MASK                0xffffff
#define MBFL_WCSPLANE_BIG5           0x70f40000              /*  2121h - 9898h */
#define MBFL_WCSGROUP_THROUGH           0x78000000              /* 000000h - FFFFFFh */
#define MBFL_WCSPLANE_MASK                      0xffff

void Viewer::Big5StringToDes(const char *aStr, TDes &aTarget)
{
  aTarget.SetLength(0);
  int status = 0;
  const char *s = aStr;
  unsigned char c1 = 0;
  int w;
  while (*s) {
    switch (status) {
      case 0:{
	unsigned char c = *s;
	  s++;
	  if (c < 0x80) { //latin
	  aTarget.Append(*s);
	  continue;
	} else if (c > 0xa0 && c < 0xff) {      /* dbcs lead byte */
	  status = 1;
	  c1 = c;
	  continue;
	} else {
	  w = c & MBFL_WCSGROUP_MASK;
	  w |= MBFL_WCSGROUP_THROUGH;
	  aTarget.Append(w);
	}	
	break;
      }
      case 1:{
	/* dbcs second byte */
	unsigned char c = *s;
	s++;
	status = 0;
	if ((c > 0x39 && c < 0x7f) | (c > 0xa0 && c < 0xff)) {
	  if (c < 0x7f){
	    w = (c1 - 0xa1)*157 + (c - 0x40);
	  } else {
	    w = (c1 - 0xa1)*157 + (c - 0xa1) + 0x3f;
	  }
	  if (w >= 0 && w < big5_ucs_table_size) {
	    w = big5_ucs_table[w];
	  } else {
	    w = 0;
	  }
	  if (w <= 0) {
	    w = (c1 << 8) | c;
	    w &= MBFL_WCSPLANE_MASK;
	    w |= MBFL_WCSPLANE_BIG5;
	  }
	  aTarget.Append(w);
	} else if (c < 0x21 || c == 0x7f) {         /* CTLs */
	  aTarget.Append(*s);
	} else {
	  w = (c1 << 8) | c;
	  w &= MBFL_WCSGROUP_MASK;
	  w |= MBFL_WCSGROUP_THROUGH;
	  aTarget.Append(w);
	}
	break;
      }
      default:
	status = 0;

    }
  }	
}


/*convert from Descriptor to C String*/
void Viewer::MyDesToString(const TDesC &aDes, char *aTarget)
{
      if (iConverterAvailability == CCnvCharacterSetConverter::EAvailable) {
        TBuf<1> unicodeBuffer;
        TBuf8<10> asciiBuffer ;
        for (int i = 0; i<aDes.Length(); i++){
            unicodeBuffer.SetLength(0);
            unicodeBuffer.Append(aDes[i]);
            iCharacterConverter->ConvertFromUnicode(asciiBuffer, unicodeBuffer);
            int j = 0;
            int len = asciiBuffer.Length();
            while (j < len) {
              *aTarget = (char)asciiBuffer[j];
              aTarget++;
              j++;
            }
        }
        *aTarget = 0;
      } else { /*fallback to ASCII*/
          DesToString(aDes, aTarget);
      }
}



/*Use long book name? (e.g. Genesis instead of Gen) */
TBool Viewer::UseLongBookName()
{
	return iLongName;	
}

/*
Set long book name flag. On SERIES_80, it is always wide enough to display everything
*/
void Viewer::SetLongBookName(TBool aLongname)
{
#ifdef SERIES_60

	iLongName = aLongname;
#else
	iLongName = 1;
#endif
}


/*is document should be rendered from right to left
 Currently only for hebrew text
 */
TBool Viewer::IsRToL()
{
	return isHebrew;
}

/*set current zoom level
  doesn't update anything but the font
  (called on startup)
*/
void Viewer::SetZoomLevel(int level)
{
	iZoomLevel = level;
	setFont(iFontNum);
}

/*get current zoom level*/
int Viewer::GetZoomLevel()
{
	return iZoomLevel;
}

/*Note: when zooming in or out then we must recalculate everything*/

/*Increase Zoom Level, if max is reached, then do nothing*/
void Viewer::ZoomIn()
{
	iZoomLevel++;
	if (iZoomLevel>=MAX_ZOOM) {
		iZoomLevel = MAX_ZOOM-1;
	}
	setFont(iFontNum); /*recalc scrolllen*/
	ClearVerseList();
	FillVerse();
}

/*Decrease  Zoom Level, if min is reached, then do nothing*/
void Viewer::ZoomOut()
{
	iZoomLevel--;
	if (iZoomLevel<0) {
		iZoomLevel = 0;
	}
	setFont(iFontNum);/*recalc scrolllen*/
	ClearVerseList();
	FillVerse();
	
}
