/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#include "KeyBindingView.h"
#include <S60Bible.rsg>
#include "S60Bible.hrh"
#include "KeyBindingList.h"
#include <barsread.h>
#include "CStringList.h"
#include "S60BibleUid.h"
#include "S60BibleDocument.h"


// factory constructor leaving instance on cleanup
CKeyBindingView* CKeyBindingView::NewLC(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CKeyBindingView* self = new (ELeave) CKeyBindingView();
	CleanupStack::PushL(self);
	self->ConstructL(aRect, aDocument);
	return self;
}

// factory constructor - no instance left on cleanup
CKeyBindingView* CKeyBindingView::NewL(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CKeyBindingView* self = CKeyBindingView::NewLC(aRect, aDocument);
	CleanupStack::Pop(self);
	return self;
}

// second phase constructor
void CKeyBindingView::ConstructL(const TRect& /*aRect*/,  CS60BibleDocument* aDocument)
{
	// construct from resource
	BaseConstructL(R_KEYBINDINGVIEW);
	iDocument = aDocument;
}

// first phase (non-leaving) constructor
CKeyBindingView::CKeyBindingView()
: CAknView()
{
}

// activation of view
void CKeyBindingView::DoActivateL(const TVwsViewId& aPrevViewId, 
										 TUid /*aCustomMessageId*/, 
								   const TDesC8& /*aCustomMessage*/)
{
	iPrevView = aPrevViewId;
	if(!iSettingList) {	// if we haven't already created iSettingList
		// construct control and set parent
		iSettingList = CKeyBindingList::NewL(this, iDocument);
		iDocument->Model()->LoadBindings();
		iSettingList->SetMopParent(this);
		
		// construct control based on resource definition
		TResourceReader reader;
		
		// CreateResourceReaderLC will allocate a buffer to be used by
		// the TResourceReader. This buffer is pushed onto the cleanup
		// stack - not the TResourceReader itself
		CEikonEnv::Static()->CreateResourceReaderLC(reader, R_KEYBINDINGSETTINGS);
		iSettingList->ConstructFromResourceL(reader);
		
		// Clean up the buffer allocated above, NOT the reader itself.
		// Cannot use expected item overload of PopAndDestroy() as buffer 
		// is inaccessible. 
		CleanupStack::PopAndDestroy();	
		
		// register view and activate
		AppUi()->AddToViewStackL(*this, iSettingList);
		iSettingList->ActivateL();
	}
}

// deactivation of view
void CKeyBindingView::DoDeactivate()
{
	if(iSettingList) {	// if setting list has been created
		// remove setting list from stack
		AppUi()->RemoveFromViewStack(*this, iSettingList);
		
		// clean up 
		delete iSettingList;
		iSettingList = NULL;
	}
}

TUid CKeyBindingView::Id() const
{
	return TUid::Uid(KKeyBindingViewID);
}

void CKeyBindingView::HandleCommandL(TInt aCommand)
{
	switch(aCommand) {
		case ESettingCmdKeyEdit :					// edit current item
			{
				iSettingList->EditCurrentItemL();
				break;
			}
		case EAknSoftkeyBack:
			{
				AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
				break;
			}
		case KSaveKeyBindings:
			{
				iSettingList->StoreSettingsL();
				iDocument->Model()->SaveBindings();
				AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
				break;
			}
		default :
			{	
				AppUi()->HandleCommandL(aCommand);
				break;
			}
	}
}


void CKeyBindingView::SizeChanged()
{
	if (iSettingList) {
		iSettingList->SetRect(ClientRect());
	}
}
