#include <e32base.h>
#include <coecobs.h>
#include <eikenv.h>
#include <eikspane.h>
#include <aknviewappui.h>
#include "abouts60.h"
#include "S60BibleUid.h"
#include <S60Bible.rsg>

CAboutView * CAboutView::NewL()
{
    	CAboutView* self = CAboutView::NewLC();
    	CleanupStack::Pop(self);
    	return self;
}


CAboutView * CAboutView::NewLC()
{
    CAboutView* self = new (ELeave) CAboutView();
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;

}

void CAboutView::ConstructL()
{
	BaseConstructL(R_ABOUTVIEW);

}

TUid CAboutView::Id() const
{
    return TUid::Uid(KAboutViewID);
}



void CAboutView::DoActivateL(const TVwsViewId& aPrevViewId,
                                    TUid /*aCustomMessageId*/,
                                    const TDesC8& /*aCustomMessage*/)
{
	ASSERT(iContainer == NULL);
    	iPrevView = aPrevViewId;
        iContainer = CAboutContainer::NewL(ClientRect());
        iContainer->SetMopParent(this);
    	AppUi()->AddToStackL(iContainer);
}

void CAboutView::DoDeactivate()
{
	if (iContainer){
		AppUi()->RemoveFromStack(iContainer);
        	delete iContainer;
        	iContainer = NULL;
        }
}


void CAboutView::HandleCommandL(TInt aCommand)
{
 	switch(aCommand){
        	case EAknSoftkeyBack:
        		{
        			AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
        			break;
			}
		default:
			break;
 	}
}

CAboutView::CAboutView()
{

}


CAboutContainer::~CAboutContainer()
{
    	delete iRichText;
    	delete iParaFormatLayer;
	delete iCharFormatLayer;
	delete iLayout;
        delete iTextView;
	delete iSBFrame;
}

CAboutContainer * CAboutContainer::NewL(const TRect &aRect)
{
	CAboutContainer* self = CAboutContainer::NewLC(aRect);
	CleanupStack::Pop(self);
	return self;
}

CAboutContainer * CAboutContainer::NewLC(const TRect &aRect)
{
    	CAboutContainer* self = new (ELeave) CAboutContainer;
    	CleanupStack::PushL(self);
    	self->ConstructL(aRect);
    	return self;
}

void CAboutContainer::ConstructL(const TRect &aRect)
{

	CreateWindowL();
	SetRect(aRect);
	TRect view = aRect;
	view.Shrink(3,3);

	ActivateL();
  	TCharFormat charFormat;
  	TCharFormatMask charFormatMask;
#ifdef __SERIES60_30__
	const CFont* logicalFont = AknLayoutUtils::FontFromId(EAknLogicalFontSecondaryFont);
	TFontSpec fontSpec = logicalFont->FontSpecInTwips();
	charFormat.iFontSpec = fontSpec;
	charFormatMask.SetAttrib(EAttFontTypeface);
	charFormatMask.SetAttrib(EAttFontHeight);
#else
  	charFormat.iFontSpec.iTypeface.iName = _L("LatinPlain12"); 	
  	charFormatMask.SetAttrib(EAttFontTypeface);
#endif
  	iParaFormatLayer=CParaFormatLayer::NewL(); // required para format layer
  	iCharFormatLayer=CCharFormatLayer::NewL(charFormat,charFormatMask); // required char format layer
  	iRichText=CRichText::NewL(iParaFormatLayer, iCharFormatLayer);
  	CParaFormat *paraFormat = CParaFormat::NewL();
  	CleanupStack::PushL(paraFormat);
  	TParaFormatMask paraFormatMask;
  	TInt len;
  	TInt pos;

  	iRichText->AppendParagraphL(11);
  	paraFormatMask.SetAttrib(EAttAlignment); // interested in alignment
  	paraFormat->iHorizontalAlignment=CParaFormat::ECenterAlign;

  	pos = iRichText->CharPosOfParagraph(len,0); // get start of first para
  	iRichText->SetInsertCharFormatL(charFormat,charFormatMask,0);
  	iRichText->ApplyParaFormatL(paraFormat,paraFormatMask,pos,1);

	HBufC* str = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_ABOUT_TITLE);
  	//_LIT(KTitle, "About S60Bible ");
	iRichText->InsertL(0, *str);


  	charFormatMask.SetAttrib(EAttColor);
  	charFormat.iFontPresentation.iTextColor=TLogicalRgb( TRgb(0,0,255) ) ; //Blue
  	charFormat.iFontSpec.iTypeface.iName = _L("LatinBold12");
  	charFormatMask.SetAttrib(EAttFontTypeface);
  	iRichText->ApplyCharFormatL(charFormat, charFormatMask, 0, str->Length());
        CleanupStack::PopAndDestroy(str);


	TBuf<128> buf;

	charFormatMask.ClearAttrib(EAttFontTypeface);
  	charFormatMask.SetAttrib(EAttColor);
  	charFormat.iFontPresentation.iTextColor=TLogicalRgb( TRgb(0,0,255) ) ; //Blue
  	charFormatMask.SetAttrib(EAttFontUnderline); // interested in underline
  	charFormat.iFontPresentation.iUnderline=EUnderlineOn; // set it on

	HBufC* email = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_ABOUT_EMAIL);
	//_LIT(KEmailAdress, "yohanes@opensource.or.id") ;


	pos = iRichText->CharPosOfParagraph(len,2);
	CEikonEnv::Static()->ReadResource(buf, R_ABOUT_LINE_1);
	TInt pos2 = pos + buf.Find(*email); // Beginning of the www adress

	iRichText->InsertL(pos,buf);
	iRichText->ApplyCharFormatL(charFormat, charFormatMask, pos2, email->Length() );
        CleanupStack::PopAndDestroy(email);

	pos = iRichText->CharPosOfParagraph(len,3);
	HBufC* url = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_ABOUT_URL);
	CEikonEnv::Static()->ReadResource(buf, R_ABOUT_LINE_2);
	iRichText->InsertL(pos,buf);

	pos = iRichText->CharPosOfParagraph(len,4);
	CEikonEnv::Static()->ReadResource(buf, R_ABOUT_LINE_3);
	pos2 = pos + buf.Find(*url); // Beginning of the www adress
	iRichText->InsertL(pos,buf);

  	iRichText->ApplyCharFormatL(charFormat, charFormatMask, pos2, url->Length() );
	CleanupStack::PopAndDestroy(url);

        HBufC* line;

	pos = iRichText->CharPosOfParagraph(len,6);
        line = CCoeEnv::Static()->AllocReadResourceLC(R_ABOUT_LINE_4);
	iRichText->InsertL(pos,*line);
        CleanupStack::PopAndDestroy(line);

        pos = iRichText->CharPosOfParagraph(len,8);
        line = CCoeEnv::Static()->AllocReadResourceLC(R_ABOUT_LINE_5);
        iRichText->InsertL(pos,*line);
        CleanupStack::PopAndDestroy(line);

        pos = iRichText->CharPosOfParagraph(len,9);
        line = CCoeEnv::Static()->AllocReadResourceLC(R_ABOUT_LINE_6);
        iRichText->InsertL(pos, *line);
        CleanupStack::PopAndDestroy(line);

        pos = iRichText->CharPosOfParagraph(len,10);
        line = CCoeEnv::Static()->AllocReadResourceLC(R_ABOUT_LINE_7);
        iRichText->InsertL(pos, *line);
        CleanupStack::PopAndDestroy(line);

        pos = iRichText->CharPosOfParagraph(len,11);
        line = CCoeEnv::Static()->AllocReadResourceLC(R_ABOUT_LINE_8);
        iRichText->InsertL(pos, *line);
        CleanupStack::PopAndDestroy(line);


	CWindowGc& gc=SystemGc(); // get graphics context
    	CBitmapDevice *device=(CBitmapDevice*) (gc.Device()); // device

    	iLayout=CTextLayout::NewL(iRichText,Rect().Width());
    	// Create text view
    	iTextView=CTextView::NewL(iLayout, Rect(),
        	device,
        	device,
        	&Window(),
        	0, // no window group
        	&iCoeEnv->WsSession()
        ); // new view

        iSBFrame = new( ELeave ) CEikScrollBarFrame( this, NULL, ETrue );
        iSBFrame->SetScrollBarVisibilityL( CEikScrollBarFrame::EOff,
                                           CEikScrollBarFrame::EOn );
        iSBFrame->SetTypeOfVScrollBar( CEikScrollBarFrame::EArrowHead );
#ifndef __SERIES60_30__
		iSBFrame->SetScrollBarManagement(CEikScrollBar::EVertical,
		CEikScrollBarFrame::EFloating);
#endif
        CleanupStack::PopAndDestroy(paraFormat);
    	iTextView->FormatTextL();
    	UpdateScrollIndicatorL();
	DrawNow();

}

TKeyResponse CAboutContainer::OfferKeyEventL( const TKeyEvent& aKeyEvent, TEventCode aType )
{
	if (aType ==  EEventKey) {
        	switch (aKeyEvent.iScanCode){
        	case EStdKeyDownArrow: {
                	TInt line = -1;
                	iTextView->ScrollDisplayLinesL(line);
                	break;
            	}

        	case EStdKeyUpArrow:  {
                	TInt line = 1;
                	iTextView->ScrollDisplayLinesL(line);
                	break;
            	}
        	}
		UpdateScrollIndicatorL();
	}

	//DrawNow();
	return CCoeControl::OfferKeyEventL(aKeyEvent, aType);
}

TInt CAboutContainer::CountComponentControls() const
{
	return 0;
}

CCoeControl* CAboutContainer::ComponentControl(TInt /*aIndex*/) const
{
	return NULL;
}

void CAboutContainer::Draw(const TRect& aRect) const
{
	TRAPD(err,iTextView->DrawL(aRect));
}

CAboutContainer::CAboutContainer()
{

}

TCoeInputCapabilities CAboutContainer::InputCapabilities() const
{
	return TCoeInputCapabilities::ENone;
}

/*taken from symbian ogg player */
void CAboutContainer::UpdateScrollIndicatorL()
    {


    TEikScrollBarModel hSbarModel;
    TEikScrollBarModel vSbarModel;

    vSbarModel.iThumbPosition = iLayout->PixelsAboveBand();
    vSbarModel.iScrollSpan = iLayout->FormattedHeightInPixels() - iLayout->BandHeight();
    vSbarModel.iThumbSpan = 1;

    TEikScrollBarFrameLayout layout;
    TRect rect( Rect() );
    iSBFrame->TileL( &hSbarModel, &vSbarModel, rect, rect, layout );
    iSBFrame->SetVFocusPosToThumbPos( vSbarModel.iThumbPosition );
    iSBFrame->DrawScrollBarsNow();
    }
