/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef READER_H
#define READER_H

#undef USE_LIB_C

#ifdef USE_LIB_C
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "usermem.h"
#else
#include <e32std.h>
#include <w32std.h>
#endif

#include "const.h"
#include "FullReader.h"
#include "Verse.h"
#include "Database.h"
#include "info.h"
#include "Decoder.h"

class Reader {
	public:
		IMPORT_C Reader();
#ifdef USE_LIB_C
		IMPORT_C void ConstructL(RFs &fsSession, const char *filename);
#else
		IMPORT_C void ConstructL(RFs &fsSession, const TDesC &filename);
#endif

		IMPORT_C ~Reader();
		IMPORT_C void Bookmark();
		IMPORT_C void Return();
		IMPORT_C bool SeekVerse(UInt16 version, UInt16 book, UInt16
                                chapter, UInt16 verse);
		IMPORT_C bool GetCurrentVerseAsString(char* verse);
		EXPORT_C bool GetCurrentTextAsString(char* verse, int type);
		IMPORT_C void MovePrevVerse();
		IMPORT_C void MoveNextVerse();
		IMPORT_C int VerseNumber();
		IMPORT_C int ChapterNumber();
		IMPORT_C int BookNumber();
		IMPORT_C char *BookName();
		IMPORT_C char *SimpleBookName();
		IMPORT_C bool SeekVerseAsString(UInt16 version, char *str);
		IMPORT_C bool SeekVerseInCurrentBook(UInt16 chapter, UInt16 verse);
		IMPORT_C void GetLastVerse(int *chapter, int *verse);
		IMPORT_C UInt16 getTotalChapter();
		IMPORT_C UInt16 getTotalVerse();
		IMPORT_C UInt16 getTotalVerse(int chapter);
		IMPORT_C int GetFirstBook();
		IMPORT_C int GetNextBook();
		IMPORT_C int GetNextBook(int book);
		IMPORT_C int GetPrevBook();
		IMPORT_C char * GetBookInfo(int book, int &chapter, int &verse);
		IMPORT_C int GetSepChar();
		IMPORT_C bool GetCurrentVerseAsStringF(char* verse,
					       UInt16* linepos,
					       UInt16* lineformat,
					       int* linecount);
		IMPORT_C int GetTotalBookChapter(int book);
		IMPORT_C int GetTotalChapterVerse(int book, int chapter);
		
	private:
		UInt32 getValue32Start(UInt32* valuePtr, Int16 index);
		UInt16 getValue16Length(UInt16* valuePtr, Int16 index);
		UInt16 getValue16Start(UInt16* valuePtr, Int16 index);
		UInt16 getVerseStartLength();
		int parseBookName(char *str, char *result);
		int parseNumber(char *str, int &chapter, int &verse);
		void openIndexRecord();
		void closeIndexRecord();
		void closeAllReader();
		VersionInfo* version_info;
		PDBAccess *pdbaccess;

        UInt16 version;
        UInt16 book;
        UInt16 chapter;
        UInt16 verse;

        UInt16 indexRecord;
        UInt16 totalRecord;

        UInt16 totalChapters;
        UInt16* totalVersesAcc;
        UInt32* totalChapterCharsAcc;
        UInt16* totalVerseCharsAcc;
        UInt32 verseStart;
        UInt16 verseLength;

        UInt16 bookmarkChapter;
        UInt16 bookmarkVerse;
		FullReader* recordReader;
		Database *dbase;
		Decoder* decoder;
};
#endif
