/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef __S60BIBLEMAINVIEW_H__
#define __S60BIBLEMAINVIEW_H__

#include <aknappui.h>
#include <coecntrl.h>
#include <e32std.h>
#include <aknappui.h>
#include <eikmenup.h>
#include <AknWaitDialog.h>
#include <aknview.h>
#include <aknquerydialog.h>
#include <aknwaitnotewrapper.h>
#include "CFileFinder.h"
#include "CFileFinderEngine.h"
#include "VerseInfo.h"
#include "S60BibleMainContainer.h"

class	CS60BibleDocument;

_LIT(KBibleListFile, "biblelist.txt");
_LIT(KSettingsFile, "biblesettings.txt");
_LIT(KOptionsFile, "bibleoptions.txt");
_LIT(KHistoryFile, "biblehistory.txt");
_LIT(KBookmarkFile, "biblebookmark.txt");
_LIT(KBindingsFile, "biblekeybindings.txt");
_LIT(KCrashFile, "crash.txt");

/**
 * CS60BibleAppView
 * Main view for s60bible
 *
 */
class CS60BibleMainView :  public CAknView,

#if 0
public MAknBackgroundProcess 
#endif
public MProgressDialogCallback
{
	public:
	/**
	 * Get the ID of this View
	 * @return UID of this View
	 */

	TUid Id() const;

	/**
	 * Second stage of 2 stage constructor
	 */
	void ConstructL();


	/**
	 * Creates a CS60BibleAppView object, which will draw itself to aRect
	 * @param aRect A rectangle that defines the size and location
	 * of the displayable area for the view
	 * @param aDocument the document
	 */
	static CS60BibleMainView* NewL(const TRect& aRect,
		CS60BibleDocument* aDocument);

	/**
	 * Creates a CS60BibleMainView object, which will draw itself to aRect
	 * @param aRect A rectangle that defines the size and location of the displayable area
	 * for the view
	 * @param aDocument the document
	 */
	static CS60BibleMainView* NewLC(const TRect& aRect,
		CS60BibleDocument* aDocument);

	/**
	 *
	 * Destroys the object
	 */
	 ~CS60BibleMainView();

public:

	/*MAknBackgroundProcess*/


#if 0	
	TInt CycleError  (  TInt  aError   );
 	TBool IsProcessDone  () const;
 	void ProcessFinished  ();
 	void StepL  ();  
#else 	
	void DialogDismissedL  (  TInt    );
#endif	

	/**
	 * show bible selection dialog
	 */
	void ShowBibleSelection();
        /**
         * show bible encoding selection dialog
         */
        void ShowEncodingSelection();
        /**
	 * show history dialog
	 */
	void ShowHistory();
	/**
	 * called when a view is being activated
	 * @param aPrevViewId previous view ID
	 * @param aCustomMessageId custom message ID
	 * @param aCustomMessage custom message
	 */
	void DoActivateL(const TVwsViewId& aPrevViewId,
				    TUid aCustomMessageId,
				    const TDesC8& aCustomMessage);
	/**
	 * Called when a view is being deactivated
	 */
	void DoDeactivate();

	/**
	 * Handle menu command
	 * @param aCommand the command
	 */
	void HandleCommandL(TInt aCommand);

public:
	/**
	 * called when this view lost its focus
	 */
	void LostFocus();
	/**
	 * called when this view gain focus
	 */
	void GainFocus();


private:
	/**
	 * Load settings from file
	 * @param aFirst true if loading setting for the first time
	 */
	void LoadSettings(TBool aFirst);
	/**
	 * First stage of 2 phase constructor
	 */
	CS60BibleMainView();
	/**
	 * First stage of 2 phase constructor
	 * @param aDocument the document
	 */
	CS60BibleMainView(CS60BibleDocument* aDocument);
	/**
	 * Performs the second phase construction
	 * @param aRect the display area for the view
	 */
	void ConstructL(const TRect& aRect);

	/**
	 * save current settings
	 */
	void SaveSettingsL();

#ifdef __SERIES60_30__
	void SearchInDir(CStringList *strlist, const TDesC& dir);
#endif

	/**
	 * Search for bible files on entire drive
	 */
	 
	void SearchForBibleFilesL();

	/**
	 * Show "goto verse" dialog
	 */
	void GotoVerse();

	void ShowFontSelection();	
	void FontNumToName(TDes& aFontName, const TInt aFontNum);
	TInt FontNameToNum(TDesC& aFontName);
	
	void LoadFontsL(const TDesC& aExtension);
	void ChooseLanguageL();
	void ShowColorSelectionL();
	
private:

	/**
	  * iDocument The document
	  */
	CS60BibleDocument* iDocument;

	/**
	 * Container for this view
	 */
    	CS60BibleMainContainer* iContainer;

	/**
	 * last verse typed for goto verse menu
	 */
    	TBuf<256> iLastVerse;
	/**
	 * backlight state
	 */
    	TBool iBackLight;
	/**
	 * fullscreen state
	 */
    	TBool iFullScreen;

	/**
	 * data path
	 */
	TFileName iDataPath;
	/**
	 * path to bible list
	 */
	TFileName iBibleList;
	/**
	 * path to bible settings
	 */
	TFileName iBibleSettings;
	/**
	 * path to bible options
	 */
	TFileName iBibleOptions;
	/**
	 * path to bible history
	 */
	TFileName iBibleHistory;
	
	TFileName iCrashFile;
	/**
	 * history content
	 */
	 
	CStringList *iHistory;
	TBool iFullBookName;
	
	TFileName tmpFontPath;
	
	TInt iOffset;
	
	CFileFinderEngine *iFinderEngine;
	/**
	 * mark that the search has been done
	 */
	TBool iSearchDone;
	
};

#endif // __S60BibleAPPVIEW_H__

