#ifndef MOBILE_EDITOR_CONTAINER
#define MOBILE_EDITOR_CONTAINER
#include <coecntrl.h>
#include <ckntitle.h>

class CS80BibleView;
class CS80BibleDocument;
class CEikRichTextEditor;
class CRichText;

class CS80BibleContainer : public MCoeControlObserver, public CEikBorderedControl
{
    public: // Constructors and destructor

        /**
        * Default constructor.
        * @param aView 
        */
        CS80BibleContainer(CS80BibleView* aView, CS80BibleDocument* aDocument);

        /**
        * EPOC constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL( const TRect& aRect );

        /**
        * Destructor.
        */
        virtual ~CS80BibleContainer();

    public: // New functions

        /**
        * Handles the commands.
        * @param aCommand command to be handled
        */
        void HandleCommandL( TInt aCommand );

    public: // From CCoeControl

        /**
        * From CoeControl,OfferKeyEventL.
        * Handles the key-events.
        * @return if key-event is consumed, EKeyWasConsumed. Else EKeyWasNotConsumed.
        * @param aKeyEvent Key event
        * @param aType Type of key event(EEventKey, EEventKeyUp or EEventKeyDown)
        */
        TKeyResponse OfferKeyEventL( const TKeyEvent& aKeyEvent,
            TEventCode aType );
        
    private: // From CCoeControl

        /**
        * From CoeControl, CountComponentControls.
        * Returns number of components.
        * @return Number of controls
        */
        TInt CountComponentControls() const;

        /**
        * From CCoeControl, ComponentControl.
        * Returns pointer to particular component.
        * @return Pointer to Control's component control, identified by index
        * @param aIndex Index of the control
        */
        CCoeControl* ComponentControl( TInt aIndex ) const;

        /**
        * From CCoeControl, Draw.
        * Clears the window's rectangle.
        * @param aRect Region of the control to be (re)drawn
        */
        void Draw( const TRect& aRect ) const;

    private: // From MCoeControlObserver
        /**
        * From MCoeControlObserver, HandleControlEventL.
        * Handles the event when state change.
        * @param aControl Pointer to control object
        * @param aEventType Event type
        */
        void HandleControlEventL(
            CCoeControl* aControl,
            TCoeEvent aEventType );    
		void SizeChanged();
    private:
            CS80BibleView *iView;
			CS80BibleDocument *iDocument;
   	public:
			CCknAppTitle* iTitlePane;
};

#endif

